@@ -2,7 +2,7 @@
 # -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-
 #   Catfish - a versatile file searching tool
 #   Copyright (C) 2007-2012 Christian Dywan <christian@twotoasts.de>
-#   Copyright (C) 2012-2020 Sean Davis <bluesabre@xfce.org>
+#   Copyright (C) 2012-2022 Sean Davis <bluesabre@xfce.org>
 #
 #   This program is free software: you can redistribute it and/or modify it
 #   under the terms of the GNU General Public License version 2, as published
@@ -37,6 +37,8 @@ import gi
 gi.require_version('GLib', '2.0')  # noqa
 from gi.repository import GLib
 
+from catfish_lib import FiletypeLists
+
 try:
     gi.require_version('Zeitgeist', '2.0')
     from gi.repository import Zeitgeist
@@ -214,8 +216,13 @@ class CatfishSearchEngine:
                         yield True
                         continue
 
-                    if method.method_name == 'fulltext' or  \
-                            zipfile.is_zipfile(filename) or \
+                    if search_zips and method.method_name == 'walk':
+                        if os.path.isfile(filename) and \
+                                zipfile.is_zipfile(filename):
+                            yield filename
+                            continue
+
+                    if method.method_name == 'fulltext' or \
                             all(key in
                                 os.path.basename(filename).lower()
                                 for key in keys):
@@ -231,7 +238,6 @@ class CatfishSearchEngine:
                         filename = filename.strip()
 
                         if len(wildcard_chunks) == 0 or \
-                                zipfile.is_zipfile(filename) or \
                                 method.method_name == 'fulltext':
                             yield filename
                             file_count += 1
@@ -270,11 +276,11 @@ class CatfishSearchEngine:
         with zipfile.ZipFile(fullpath, 'r') as z:
             for member in z.infolist():
                 for method in self.methods:
-                    if method.method_name is 'walk':
+                    if method.method_name == 'walk':
                         if self.search_filenames(
                                 member.filename, keywords, search_exact):
                             yield (member.filename, member.file_size, member.date_time)
-                    if method.method_name is 'fulltext':
+                    if method.method_name == 'fulltext':
                         if method.search_zip(z, member, keyword_list):
                             yield (member.filename, member.file_size, member.date_time)
 
@@ -285,12 +291,11 @@ class CatfishSearchEngine:
             if " ".join(keywords) in fname.lower():
                 return True
         else:
-            match_list = set()
-            for kword in keywords:
-                if kword in fname.lower():
-                    match_list.add(kword)
-                if len(set(keywords)) == len(match_list):
-                    return True
+            fname = fname.lower()
+            for kword in set(keywords):
+                if kword not in fname:
+                    return False
+            return True
 
     def set_exact(self, exact):
         """Set method for exact"""
@@ -341,7 +346,7 @@ class CatfishSearchMethod_Walk(CatfishSearchMethod):
 
     def __init__(self):
         """Initialize the 'walk' Search Method."""
-        CatfishSearchMethod.__init__(self, "walk")
+        super().__init__("walk")
         self.running = False
 
     def get_dir_list(self, root, dirs, xdg_list,
@@ -420,7 +425,8 @@ class CatfishSearchMethod_Walk(CatfishSearchMethod):
             GLib.get_user_special_dir(GLib.UserDirectory.DIRECTORY_DOWNLOAD),
             GLib.get_user_special_dir(GLib.UserDirectory.DIRECTORY_MUSIC),
             GLib.get_user_special_dir(GLib.UserDirectory.DIRECTORY_PICTURES),
-            GLib.get_user_special_dir(GLib.UserDirectory.DIRECTORY_PUBLIC_SHARE),
+            GLib.get_user_special_dir(
+                GLib.UserDirectory.DIRECTORY_PUBLIC_SHARE),
             GLib.get_user_special_dir(GLib.UserDirectory.DIRECTORY_TEMPLATES),
             GLib.get_user_special_dir(GLib.UserDirectory.DIRECTORY_VIDEOS),
         ]
@@ -484,7 +490,7 @@ class CatfishSearchMethod_Fulltext(CatfishSearchMethod):
 
     def __init__(self):
         """Initialize the 'fulltext' search method."""
-        CatfishSearchMethod.__init__(self, "fulltext")
+        super().__init__("fulltext")
         self.force_stop = False
         self.running = False
         self.exact = False
@@ -526,16 +532,7 @@ class CatfishSearchMethod_Fulltext(CatfishSearchMethod):
     def is_txt(self, filename):
         """Checks if text mimetype."""
         mime = str(mimetypes.guess_type(filename)[0])
-        text_list = ('ardour', 'audacity', 'desktop', 'document',
-                     'fontforge', 'java', 'json', 'm4', 'mbox',
-                     'message', 'mimearchive', 'msg', 'none', 'perl',
-                     'pgp-keys', 'php', 'postscript', 'rtf',
-                     'ruby', 'shellscript', 'spreadsheet', 'sql',
-                     'subrip', 'text', 'troff', 'url', 'winhlp',
-                     'x-bittorent', 'x-cue', 'x-extension-cfg',
-                     'x-glade', 'x-mpegurl', 'x-sami', 'x-theme',
-                     'x-trash', 'xml', 'xpixmap', 'yaml')
-        for filetype in text_list:
+        for filetype in FiletypeLists.text_list():
             if filetype in mime.lower():
                 return True
 
@@ -548,15 +545,18 @@ class CatfishSearchMethod_Fulltext(CatfishSearchMethod):
     def search_text(self, lines, keywords):
         if self.exact:
             for line in lines:
-                if " ".join(keywords) in line:
+                if " ".join(keywords) in line.lower():
                     return True
         else:
-            match_list = set()
+            keywords = list(set(keywords))
+            match_found = [False for i in range(len(keywords))]
+            match_found_count = 0
             for line in lines:
-                for kword in keywords:
-                    if kword in line.lower():
-                        match_list.add(kword)
-                if len(set(keywords)) == len(match_list):
+                for i in range(len(keywords)):
+                    if (not match_found[i]) and keywords[i] in line.lower():
+                        match_found[i] = True
+                        match_found_count += 1
+                if match_found_count == len(keywords):
                     return True
 
     def search_zip(self, z, member, keywords):
@@ -654,7 +654,7 @@ class CatfishSearchMethod_Zeitgeist(CatfishSearchMethod):
 
     def __init__(self):
         """Initialize the Zeitgeist SearchMethod."""
-        CatfishSearchMethod.__init__(self, "zeitgeist")
+        super().__init__("zeitgeist")
         self.stop_search = False
         self.events = []
 
@@ -669,7 +669,7 @@ class CatfishSearchMethod_Zeitgeist(CatfishSearchMethod):
 
     def run(self, keywords, path, search_zips, regex=False, exclude_paths=[]):
         """Run the Zeitgeist SearchMethod."""
-        keywords = " ".join(keywords)
+        keywords = " ".join(keywords).lower()
         self.stop_search = False
         event_template = Zeitgeist.Event()
 
@@ -700,7 +700,7 @@ class CatfishSearchMethod_Zeitgeist(CatfishSearchMethod):
                 if uri.startswith('file://'):
                     fullname = str(uri[7:])
                     filepath, filename = os.path.split(fullname)
-                    if keywords.lower() in filename and \
+                    if keywords in filename and \
                             uri not in uniques and \
                             path in filepath:
                         uniques.append(uri)
@@ -722,7 +722,7 @@ class CatfishSearchMethodExternal(CatfishSearchMethod):
 
     def __init__(self, method_name):
         """Initialize the external method class."""
-        CatfishSearchMethod.__init__(self, method_name)
+        super().__init__(method_name)
         self.pid = -1
         self.command = []
         self.process = None
@@ -787,7 +787,7 @@ class CatfishSearchMethod_Locate(CatfishSearchMethodExternal):
 
     def __init__(self):
         """Initialize the Locate SearchMethod."""
-        CatfishSearchMethodExternal.__init__(self, "locate")
+        super().__init__("locate")
         self.caps = self.get_capabilities()
         if self.caps["existing"]:
             self.command = ["locate", "-i", "%path*%keywords*", "--existing"]
